#!/usr/bin/env python3

# Copyright 2019 The Kapitan Authors
# SPDX-FileCopyrightText: 2020 The Kapitan Authors <kapitan-admins@googlegroups.com>
#
# SPDX-License-Identifier: Apache-2.0

"external input tests"
import os
import sys
import tempfile
import unittest
import string
import random
import stat
import subprocess

import yaml
from kapitan.cached import reset_cache
from kapitan.cli import main
from kapitan.inputs.external import External


class ExternalInputTest(unittest.TestCase):
    def setUp(self):
        os.chdir(os.path.join("tests", "test_resources"))

    def test_compile(self):
        temp = tempfile.mkdtemp()
        sys.argv = ["kapitan", "compile", "--output-path", temp, "-t", "external-test"]
        main()
        self.assertTrue(
            os.path.isfile(
                os.path.join(
                    temp,
                    "compiled",
                    "external-test",
                    "test.md",
                )
            )
        )

    def test_compile_file(self):
        # tests setting of args, env_vars and "\${compiled_target_dir}" being properly substituted
        external_script_content = """
        #!/usr/bin/env bash
        set -ex

        name=$1
        compiled_target_dir=$2

        echo "test-${NAME}" > "${compiled_target_dir}/${name}"
        """
        compile_path = tempfile.mkdtemp()
        temp_dir = tempfile.mkdtemp()
        external_script_file_path = os.path.join(temp_dir, "script.sh")
        with open(external_script_file_path, "w") as f:
            f.write(external_script_content)

        search_paths = [external_script_file_path]
        ref_controller = None

        external_compiler = External(compile_path, search_paths, ref_controller)

        letters = string.ascii_lowercase
        name = "".join(random.choice(letters) for i in range(8))
        external_compiler.set_args([name, "\${compiled_target_dir}"])
        external_compiler.set_env_vars({"NAME": name})

        with self.assertRaises(ValueError) as e:
            external_compiler.compile_file(external_script_file_path, compile_path, None)

        self.assertTrue("Executing external input with command" in e.exception.args[0])
        self.assertTrue("failed" in e.exception.args[0])
        self.assertTrue("Permission denied" in e.exception.args[0])

        # make file executable so there's no failure
        subprocess.check_call(["chmod", "+x", external_script_file_path])
        external_compiler.compile_file(external_script_file_path, compile_path, None)

        generated_file = os.path.join(compile_path, name)
        with open(generated_file, "r") as f:
            generated_file_content = f.readlines()[0]
        self.assertTrue(
            os.path.isfile(generated_file),
            msg="File {} should be generated by compile_file".format(generated_file),
        )
        self.assertEqual(
            generated_file_content, "test-{}\n".format(name), msg="Correct content should be written to file"
        )

    def tearDown(self):
        os.chdir("../../")
        reset_cache()



import os
import sys
import unittest
import tempfile
from shutil import rmtree, copytree
from distutils.dir_util import copy_tree
from kapitan.cached import reset_cache
from kapitan.cli import main
import yaml
from kapitan.resources import yaml_load
import json
from kapitan.dependency_manager.base import (
    fetch_git_dependency,
    fetch_http_dependency,
    fetch_git_source,
    fetch_http_source,
)


class RemoteInventoryTest(unittest.TestCase):
    def setUp(self):
        os.chdir(
            os.path.join(
                os.path.abspath(os.path.dirname(__file__)), "test_remote_inventory", "environment_one"
            )
        )

    def test_fetch_git_inventory(self):
        """
        test fetching of git inventories
        """
        temp_dir = tempfile.mkdtemp()
        repo_dir = os.path.join(temp_dir, "7a8f3940kapitan.git")
        git_source = "https://github.com/kapicorp/kapitan.git"
        fetch_git_source(git_source, repo_dir, item_type="Inventory")
        self.assertTrue(os.path.isfile(os.path.join(repo_dir, "README.md")))
        rmtree(temp_dir)

    def test_clone_inv_subdir(self):
        """
        test clonning of inventories from git and storing it in the output path
        """
        temp_dir = tempfile.mkdtemp()
        output_dir = tempfile.mkdtemp()
        git_source = "https://github.com/kapicorp/kapitan.git"
        inv = [{"output_path": os.path.join(output_dir, "subdir"), "ref": "master", "subdir": "tests"}]
        fetch_git_dependency((git_source, inv), temp_dir, force=False, item_type="inventory")
        self.assertTrue(os.path.isdir(os.path.join(output_dir, "subdir")))
        rmtree(output_dir)
        rmtree(temp_dir)

    def test_fetch_http_inventory(self):
        """
        test fetching of inventories over http
        """
        temp_dir = tempfile.mkdtemp()

        http_sources = [
            ("https://github.com/kapicorp/kapitan/archive/master.zip", "009a21cfmaster.zip"),
            (
                "https://raw.githubusercontent.com/kapicorp/kapitan/master/tests/test_resources/inventory/classes/common.yml",
                "eac6ceb7common.yml",
            ),
            (
                "https://github.com/kapicorp/kapitan/raw/master/tests/test_remote_inventory/zipped_inventories/inventory.zip",
                "47c29a3binventory.zip",
            ),
        ]
        for source, path_hash in http_sources:
            fetch_http_source(source, os.path.join(temp_dir, path_hash), item_type="Inventory")

        self.assertTrue(os.path.isfile(os.path.join(temp_dir, "009a21cfmaster.zip")))
        self.assertTrue(os.path.isfile(os.path.join(temp_dir, "eac6ceb7common.yml")))
        self.assertTrue(os.path.isfile(os.path.join(temp_dir, "47c29a3binventory.zip")))
        rmtree(temp_dir)

    def test_unpack_http_inv(self):
        """
        test unpacking of remote inventories after downloading it over http
        """
        temp_dir = tempfile.mkdtemp()
        output_dir = tempfile.mkdtemp()
        # tar.gz file
        http_source = "https://github.com/kapicorp/kapitan/raw/master/tests/test_remote_inventory/zipped_inventories/inventory.tar.gz"
        inv = [{"output_path": os.path.join(output_dir, "subdir"), "unpack": "True"}]
        fetch_http_dependency((http_source, inv), temp_dir, force=False, item_type="inventory")

        self.assertTrue(os.path.isdir(os.path.join(output_dir, "subdir", "targets")))
        rmtree(temp_dir)
        rmtree(output_dir)

    def test_compile_fetch(self):
        """Run $ kapitan compile --fetch --output-path=some/dir/ --inventory-path=another/dir --targets remoteinv-example remoteinv-nginx zippedinv --force
        were some/dir/ & another/dir/ are directories chosen by the user

        Recursively force fetch inventories and compile
        """
        temp_output = tempfile.mkdtemp()
        temp_inv = tempfile.mkdtemp()

        copy_tree(
            os.path.join(
                os.path.abspath(os.path.dirname(__file__)),
                "test_remote_inventory",
                "environment_one",
                "inventory",
            ),
            temp_inv,
        )
        sys.argv = [
            "kapitan",
            "compile",
            "--fetch",
            "--output-path",
            temp_output,
            "--inventory-path",
            temp_inv,
            "--targets",
            "remoteinv-example",
            "remoteinv-nginx",
            "zippedinv",
            "--force",
        ]
        main()

        self.assertTrue(os.path.isfile(os.path.join(temp_inv, "targets", "remoteinv-nginx.yml")))
        self.assertTrue(os.path.isfile(os.path.join(temp_inv, "targets", "nginx.yml")))
        self.assertTrue(os.path.isfile(os.path.join(temp_inv, "targets", "nginx-dev.yml")))
        self.assertTrue(os.path.isdir(os.path.join(temp_output, "compiled", "remoteinv-nginx")))
        self.assertTrue(os.path.isdir(os.path.join(temp_output, "compiled", "zippedinv")))

        rmtree(temp_inv)
        rmtree(temp_output)

    def test_compile_fetch_classes_that_doesnot_exist_yet(self):
        """
        runs $ kapitan compile --fetch --search-paths temp_dir --output-path temp_dir --inventory-path temp_dir/inventory -t nginx
        The `test-objects` class of target `nginx` does not exist initially, it is fetched and then compiled
        """
        os.chdir(os.path.join(os.getcwd(), "..", "environment_three"))
        temp_dir = tempfile.mkdtemp()

        copy_tree(".", temp_dir)  # copying the test environment to search path
        sys.argv = [
            "kapitan",
            "compile",
            "--fetch",
            "--search-path",
            temp_dir,
            "--output-path",
            temp_dir,
            "--inventory-path",
            os.path.join(temp_dir, "inventory"),
            "-t",
            "nginx",
        ]
        main()
        self.assertTrue(os.path.exists(os.path.join(temp_dir, "compiled", "nginx")))
        rmtree(temp_dir)

    def test_force_fetch(self):
        """Test overwriting inventory items while using the --force flag

        runs $ kapitan compile --fetch --cache --output-path=temp_output --inventory-pat=temp_inv --targets remoteinv-example
        runs $ kapitan compile --fetch --cache --output-path=temp_output --inventory-pat=temp_inv --targets remoteinv-example --force
        """

        temp_output = tempfile.mkdtemp()
        temp_inv = tempfile.mkdtemp()
        copy_tree(
            os.path.join(
                os.path.abspath(os.path.dirname(__file__)),
                "test_remote_inventory",
                "environment_one",
                "inventory",
            ),
            temp_inv,
        )
        sys.argv = [
            "kapitan",
            "compile",
            "--fetch",
            "--cache",
            "--output-path",
            temp_output,
            "--inventory-path",
            temp_inv,
            "--targets",
            "remoteinv-example",
        ]
        main()

        fname = os.path.join(temp_inv, "targets", "zippedinv.yml")
        data = json.loads(yaml_load([os.path.dirname(fname)], os.path.basename(fname)))
        data["parameters"]["compression_type"] = "test"
        with open(fname, "w") as yaml_file:
            yaml_file.write(yaml.dump(data, default_flow_style=False))

        reset_cache()
        main()
        # no change in zippedinv.yml
        new_data = json.loads(yaml_load([os.path.dirname(fname)], os.path.basename(fname)))

        reset_cache()
        sys.argv = [
            "kapitan",
            "compile",
            "--fetch",
            "--cache",
            "--output-path",
            temp_output,
            "--inventory-path",
            temp_inv,
            "--targets",
            "remoteinv-example",
            "--force",
        ]
        main()
        # zippedinv.yml overwritten
        force_fetched_data = json.loads(yaml_load([os.path.dirname(fname)], os.path.basename(fname)))
        self.assertEqual(new_data["parameters"]["compression_type"], "test")
        self.assertEqual(force_fetched_data["parameters"]["compression_type"], "gzip")
        rmtree(temp_inv)
        rmtree(temp_output)

    def tearDown(self):
        os.chdir("../../../")
        reset_cache()



#!/usr/bin/env python3

# Copyright 2019 The Kapitan Authors
# SPDX-FileCopyrightText: 2020 The Kapitan Authors <kapitan-admins@googlegroups.com>
#
# SPDX-License-Identifier: Apache-2.0

import multiprocessing
import os
import sys
import tempfile
import unittest
from shutil import rmtree

from kapitan.errors import HelmFetchingError
from kapitan.cached import reset_cache
from kapitan.cli import main
from kapitan.dependency_manager.base import (
    fetch_git_source,
    fetch_http_source,
    fetch_git_dependency,
    fetch_helm_chart,
    fetch_dependencies,
    HelmSource,
)


class DependencyManagerTest(unittest.TestCase):
    def setUp(self):
        os.chdir(os.path.join(os.getcwd(), "tests", "test_resources"))

    def test_fetch_http_sources(self):
        "Tests fetching http[s] sources"
        temp_dir = tempfile.mkdtemp()
        http_sources = [
            (
                "https://raw.githubusercontent.com/kapicorp/kapitan/master/examples/docker/components/jsonnet/jsonnet.jsonnet",
                "1c3a08e6jsonnet.jsonnet",
            ),
            (
                "https://raw.githubusercontent.com/kapicorp/kapitan/master/examples/docker/components/kadet/__init__.py",
                "aff45ec8__init__.py",
            ),
        ]

        for source, path_hash in http_sources:
            fetch_http_source(source, os.path.join(temp_dir, path_hash), item_type="Dependency")

        self.assertTrue(os.path.isfile(os.path.join(temp_dir, "1c3a08e6" + "jsonnet.jsonnet")))
        self.assertTrue(os.path.isfile(os.path.join(temp_dir, "aff45ec8" + "__init__.py")))
        rmtree(temp_dir)

    def test_fetch_git_sources(self):
        "Tests clonning git repo"
        temp_dir = tempfile.mkdtemp()
        repo_dir = os.path.join(temp_dir, "7a8f3940kapitan.git")
        # TODO: also test git ssh urls
        git_source = "https://github.com/kapicorp/kapitan.git"
        fetch_git_source(git_source, repo_dir, item_type="Dependency")
        self.assertTrue(os.path.isfile(os.path.join(repo_dir, "README.md")))
        rmtree(temp_dir)

    def test_clone_repo_subdir(self):
        """
        Tests clonning git repo and copy its' subdir
        """
        temp_dir = tempfile.mkdtemp()
        output_dir = tempfile.mkdtemp()
        source = "https://github.com/kapicorp/kapitan.git"
        dep = [
            {
                "output_path": os.path.join(output_dir, "subdir"),
                "ref": "master",
                "subdir": "tests",
            }
        ]
        fetch_git_dependency((source, dep), temp_dir, force=False)
        self.assertTrue(os.path.isdir(os.path.join(output_dir, "subdir")))
        rmtree(temp_dir)
        rmtree(output_dir)

    def test_fetch_helm_chart(self):
        """
        Tests fetching helm chart
        """
        temp_dir = tempfile.mkdtemp()
        output_dir = tempfile.mkdtemp()
        output_chart_dir = os.path.join(output_dir, "charts", "prometheus")
        chart_name = "prometheus"
        version = "11.3.0"
        repo = "https://github.com/BurdenBear/kube-charts-mirror/raw/master/docs/"
        dep = [
            {
                "output_path": output_chart_dir,
                "version": version,
                "chart_name": chart_name,
                "source": repo,
            }
        ]
        fetch_helm_chart((HelmSource(repo, chart_name, version, None), dep), temp_dir, force=False)
        self.assertTrue(os.path.isdir(output_chart_dir))
        self.assertTrue(os.path.isfile(os.path.join(output_chart_dir, "Chart.yaml")))
        self.assertTrue(os.path.isdir(os.path.join(output_chart_dir, "charts", "kube-state-metrics")))
        rmtree(temp_dir)
        rmtree(output_dir)

    def test_fetch_helm_chart_version_that_does_not_exist(self):
        """
        Test fetching helm chart version that does not exist
        """
        temp_dir = tempfile.mkdtemp()
        output_dir = tempfile.mkdtemp()
        output_chart_dir = os.path.join(output_dir, "charts", "prometheus")
        chart_name = "prometheus"
        version = "10.7.0"
        repo = "https://github.com/BurdenBear/kube-charts-mirror/raw/master/docs/"
        dep = [
            {
                "output_path": output_chart_dir,
                "version": version,
                "chart_name": chart_name,
                "source": repo,
            }
        ]
        with self.assertRaises(HelmFetchingError):
            fetch_helm_chart((HelmSource(repo, chart_name, version, None), dep), temp_dir, force=False)
        self.assertFalse(os.path.isdir(output_chart_dir))
        self.assertFalse(os.path.isfile(os.path.join(output_chart_dir, "Chart.yaml")))
        rmtree(temp_dir)
        rmtree(output_dir)

    def test_fetch_dependencies_unpack_parallel(self):
        output_path = tempfile.mkdtemp()
        save_dir = tempfile.mkdtemp()
        # use default parallelism of 4 for test
        pool = multiprocessing.Pool(4)
        target_objs = [
            {
                "dependencies": [
                    {
                        "type": "https",
                        "source": "https://github.com/BurdenBear/kube-charts-mirror/raw/master/docs/nfs-client-provisioner-1.2.8.tgz",
                        "output_path": "nfs-client-provisioner",
                        "unpack": True,
                    },
                    {
                        "type": "https",
                        "source": "https://github.com/BurdenBear/kube-charts-mirror/raw/master/docs/prometheus-pushgateway-1.2.13.tgz",
                        "output_path": "prometheus-pushgateway",
                        "unpack": True,
                    },
                ]
            }
        ]
        try:
            fetch_dependencies(output_path, target_objs, save_dir, False, pool)
            pool.close()
        except Exception as e:
            pool.terminate()
            raise e
        finally:
            pool.join()

        for obj in target_objs[0]["dependencies"]:
            self.assertTrue(os.path.isdir(os.path.join(output_path, obj["output_path"])))
            self.assertTrue(os.path.isdir(os.path.join(save_dir, obj["output_path"])))
        rmtree(output_path)
        rmtree(save_dir)

    def test_compile_fetch(self):
        "Runs $ kapitan compile --fetch --output-path temp -t nginx nginx-dev monitoring-dev"
        temp = tempfile.mkdtemp()
        sys.argv = [
            "kapitan",
            "compile",
            "--fetch",
            "--output-path",
            temp,
            "-t",
            "nginx",
            "nginx-dev",
            "monitoring-dev",
        ]
        main()
        self.assertTrue(os.path.isdir(os.path.join(temp, "components", "tests")))
        self.assertTrue(os.path.isdir(os.path.join(temp, "components", "acs-engine-autoscaler")))
        self.assertTrue(os.path.isdir(os.path.join(temp, "components", "kapitan-repository")))
        self.assertTrue(os.path.isdir(os.path.join(temp, "components", "source")))
        self.assertTrue(os.path.isdir(os.path.join(temp, "charts", "prometheus")))
        rmtree(temp)

    def tearDown(self):
        os.chdir("../../")
        reset_cache()



"Azure secrets test"

import os
import tempfile
import unittest
import io
import sys
import contextlib

from kapitan import cached
from kapitan.cli import main
from kapitan.refs.base import RefController, RefParams, Revealer
from kapitan.refs.secrets.azkms import AzureKMSSecret, AzureKMSError

REFS_HOME = tempfile.mkdtemp()
REF_CONTROLLER = RefController(REFS_HOME)
REVEALER = Revealer(REF_CONTROLLER)
REF_CONTROLLER_EMBEDDED = RefController(REFS_HOME, embed_refs=True)
REVEALER_EMBEDDED = Revealer(REF_CONTROLLER_EMBEDDED)


class AzureKMSTest(unittest.TestCase):
    "Test Azure key vault secrets"

    def test_azkms_write_reveal(self):
        """
        Write secret, confirm secret file exists, reveal and compare content
        """
        tag = "?{azkms:secret/test}"
        REF_CONTROLLER[tag] = AzureKMSSecret("mock", "mock")
        self.assertTrue(os.path.isfile(os.path.join(REFS_HOME, "secret/test")))

        file_with_secret_tags = tempfile.mktemp()
        with open(file_with_secret_tags, "w") as fp:
            fp.write("I am a ?{azkms:secret/test} value")
        revealed = REVEALER.reveal_raw_file(file_with_secret_tags)
        self.assertEqual("I am a mock value", revealed)

    def test_azkms_write_embedded_reveal(self):
        """
        write and compile embedded secret, confirm secret file exists, reveal and compare content"
        """
        tag = "?{azkms:secret/test}"
        REF_CONTROLLER_EMBEDDED[tag] = AzureKMSSecret("mock", "mock")
        self.assertTrue(os.path.isfile(os.path.join(REFS_HOME, "secret/test")))
        ref_obj = REF_CONTROLLER_EMBEDDED[tag]

        file_with_secret_tags = tempfile.mktemp()
        with open(file_with_secret_tags, "w") as fp:
            fp.write(f"I am a {ref_obj.compile()} value")

        revealed = REVEALER_EMBEDDED.reveal_raw_file(file_with_secret_tags)
        self.assertEqual("I am a mock value", revealed)

    def test_cli_secret_write_reveal_azkms(self):
        """
        run $ kapitan refs --write azkms:test_secret
        and $ kapitan refs --reveal
        using mock key
        """
        test_secret_content = "mock"
        test_secret_file = tempfile.mktemp()
        with open(test_secret_file, "w") as fp:
            fp.write(test_secret_content)

        sys.argv = [
            "kapitan",
            "refs",
            "--write",
            "azkms:test_secret",
            "-f",
            test_secret_file,
            "--refs-path",
            REFS_HOME,
            "--key",
            "mock",
        ]

        main()
        test_tag_content = "revealing: ?{azkms:test_secret}"
        test_tag_file = tempfile.mktemp()
        with open(test_tag_file, "w") as fp:
            fp.write(test_tag_content)
        sys.argv = ["kapitan", "refs", "--reveal", "-f", test_tag_file, "--refs-path", REFS_HOME]

        stdout = io.StringIO()
        with contextlib.redirect_stdout(stdout):
            main()
        self.assertEqual(f"revealing: {test_secret_content}", stdout.getvalue())

        os.remove(test_tag_file)

    def tearDown(self):
        cached.reset_cache()



import logging
import os
import tempfile

import yaml

from kapitan.errors import HelmTemplateError
from kapitan.helm_cli import helm_cli
from kapitan.inputs.base import InputType, CompiledFile

logger = logging.getLogger(__name__)

HELM_DENIED_FLAGS = {
    "dry-run",
    "generate-name",
    "help",
    "output-dir",
    "show-only",
}


class Helm(InputType):
    def __init__(self, compile_path, search_paths, ref_controller, args):
        super().__init__("helm", compile_path, search_paths, ref_controller)

        self.helm_values_files = args.get("helm_values_files")
        self.helm_params = args.get("helm_params") or {}
        self.helm_path = args.get("helm_path")
        self.file_path = None

        self.helm_values_file = None
        if "helm_values" in args:
            """dump helm values into a yaml file whose path will be passed over to Go helm code"""
            _, self.helm_values_file = tempfile.mkstemp(".helm_values.yml", text=True)
            with open(self.helm_values_file, "w") as fp:
                yaml.safe_dump(args["helm_values"], fp)

        self.kube_version = None
        if "kube_version" in args:
            logger.warning("passing kube_version is deprecated. Use api_versions helm flag instead.")
            self.kube_version = args["kube_version"]

    def compile_file(self, file_path, compile_path, ext_vars, **kwargs):
        """
        Render templates in file_path/templates and write to compile_path.
        file_path must be a directory containing helm chart.
        kwargs:
            reveal: default False, set to reveal refs on compile
            target_name: default None, set to current target being compiled
        """
        reveal = kwargs.get("reveal", False)
        target_name = kwargs.get("target_name", None)

        if self.file_path is not None:
            raise CompileError(
                "The same helm input was compiled with different input paths, which will give a wrong result."
                + f" The input paths found are: {self.file_path} and {file_path}."
                + f" The search paths were: {self.search_paths}."
            )
        self.file_path = file_path

        temp_dir = tempfile.mkdtemp()
        os.makedirs(os.path.dirname(compile_path), exist_ok=True)
        # save the template output to temp dir first
        error_message = self.render_chart(
            chart_dir=file_path,
            output_path=temp_dir,
            helm_path=self.helm_path,
            helm_params=self.helm_params,
            helm_values_file=self.helm_values_file,
            helm_values_files=self.helm_values_files,
        )
        if error_message:
            raise HelmTemplateError(error_message)

        walk_root_files = os.walk(temp_dir)
        for current_dir, _, files in walk_root_files:
            for file in files:  # go through all the template files
                rel_dir = os.path.relpath(current_dir, temp_dir)
                rel_file_name = os.path.join(rel_dir, file)
                full_file_name = os.path.join(current_dir, file)
                with open(full_file_name, "r") as f:
                    item_path = os.path.join(compile_path, rel_file_name)
                    os.makedirs(os.path.dirname(item_path), exist_ok=True)
                    with CompiledFile(
                        item_path,
                        self.ref_controller,
                        mode="w",
                        reveal=reveal,
                        target_name=target_name,
                    ) as fp:
                        yml_obj = list(yaml.safe_load_all(f))
                        fp.write_yaml(yml_obj)
                        logger.debug("Wrote file %s to %s", full_file_name, item_path)

        self.helm_values_file = None  # reset this
        self.helm_params = {}
        self.helm_values_files = []

    def default_output_type(self):
        return None

    def render_chart(
        self, chart_dir, output_path, helm_path, helm_params, helm_values_file, helm_values_files
    ):
        args = ["template"]

        name = helm_params.pop("name", None)
        output_file = helm_params.pop("output_file", None)

        flags = {"--include-crds": True, "--skip-tests": True}

        if self.kube_version:
            flags["--api-versions"] = self.kube_version

        for param, value in helm_params.items():
            if len(param) == 1:
                raise ValueError(f"invalid helm flag: '{param}'. helm_params supports only long flag names")

            if "-" in param:
                raise ValueError(f"helm flag names must use '_' and not '-': {param}")

            param = param.replace("_", "-")

            if param in ("set", "set-file", "set-string"):
                raise ValueError(
                    f"helm '{param}' flag is not supported. Use 'helm_values' to specify template values"
                )

            if param == "values":
                raise ValueError(
                    f"helm '{param}' flag is not supported. Use 'helm_values_files' to specify template values files"
                )

            if param in HELM_DENIED_FLAGS:
                raise ValueError(f"helm flag '{param}' is not supported.")

            flags[f"--{param}"] = value

        # 'release_name' used to be the "helm template" [NAME] parameter.
        # For backward compatibility, assume it is the '--release-name' flag only if its value is a bool.
        release_name = flags.get("--release-name")
        if release_name is not None and not isinstance(release_name, bool):
            logger.warning(
                "using 'release_name' to specify the output name is deprecated. Use 'name' instead"
            )
            del flags["--release-name"]
            # name is used in place of release_name if both are specified
            name = name or release_name

        for flag, value in flags.items():
            # boolean flag should be passed when present, and omitted when not specified
            if isinstance(value, bool):
                if value:
                    args.append(flag)
            else:
                args.append(flag)
                args.append(str(value))

        """renders helm chart located at chart_dir, and stores the output to output_path"""
        if helm_values_file:
            args.append("--values")
            args.append(helm_values_file)

        if helm_values_files:
            for file_name in helm_values_files:
                args.append("--values")
                args.append(file_name)

        if not output_file:
            args.append("--output-dir")
            args.append(output_path)

        if "name_template" not in flags:
            args.append(name or "--generate-name")

        # uses absolute path to make sure helm interprets it as a
        # local dir and not a chart_name that it should download.
        args.append(chart_dir)

        if output_file:
            with open(os.path.join(output_path, output_file), "wb") as f:
                # can't be verbose when capturing stdout
                return helm_cli(helm_path, args, stdout=f)
        else:
            return helm_cli(helm_path, args, verbose="--debug" in flags)
